import type { BasicAuthInfo, RefreshAuthInfo, SsoAuthInfo } from '../auth/auth.types';
import { Session } from '../session';
import type { BasicCredentialsInput, CredentialData, GetCredentialOptions, RefreshCredentialsInput, SSOCredentialsInput } from './credentialManager.types';
import { CredentialManagerConfig } from './credentialManager.types';
/**
 * The default separator used for splitting credential parts.
 * @public
 */
export declare const credentialSeparator = ":";
/**
 * The default request options for the CredentialManager service.
 * @public
 */
export declare const defaultCredentialRequestOptions: GetCredentialOptions;
/**
 * The default configuration for the CredentialManager service.
 * @public
 */
export declare const defaultCredentialManagerConfig: CredentialManagerConfig;
/**
 * Represents a system for managing user credentials and authentication tokens.
 * @public
 */
export interface CredentialManager {
    /**
     * Creates a set of basic authentication credentials using the specified input data.
     * @public
     * @param inputs - The input data to use when creating the credentials.
     * @returns A set of basic authentication credentials (username/password).
     */
    createBasicCredentials(inputs: BasicCredentialsInput): BasicAuthInfo;
    /**
     * Creates a set of refresh authentication credentials using the specified input data.
     * @public
     * @param inputs - The custom input data to use when creating the credentials, if any.
     * @returns A set of refresh authentication credentials.
     */
    createRefreshCredentials(inputs?: RefreshCredentialsInput): RefreshAuthInfo;
    /**
     * Creates a set of SSO authentication credentials using the specified input data.
     * @public
     * @param inputs - The custom input data to use when creating the credentials, if any.
     * @returns A set of SSO authentication credentials.
     */
    createSSOCredentials(inputs?: SSOCredentialsInput): SsoAuthInfo;
    /**
     * Validates the specified set of basic authentication credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateBasicCredentials(credentials: BasicAuthInfo): boolean;
    /**
     * Validates the specified set of refresh authentication credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateRefreshCredentials(credentials: RefreshAuthInfo): boolean;
    /**
     * Validates the specified set of SSO authentication credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateSSOCredentials(credentials: SsoAuthInfo): boolean;
    /**
     * Validate user credential data returned from the container or a cookie.
     *
     * @remarks
     * This can be called to ensure the credential is in the format this version of the manager expects.
     *
     * @privateRemarks
     * This can occur if the credential was saved using custom App code or via an early version of the manager itself.
     * The credential may be missing certain field values.
     *
     * @public
     * @param data - The credential data to validate.
     * @param ensurePassword - Check for the existence of a password. PasswordCredentials will have this, but cookies are considered valid without.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateUserCredentialData(data: CredentialData, ensurePassword?: boolean): boolean;
    /**
     * Normalise legacy credential data.
     *
     * @remarks
     * Older applications may have legacy credential data that needs migrating to the latest format the manager expects.
     * If the input CredentialData is already formatted as such, it will be returned as is. Otherwise, we check if any
     * cookie input is already formatted as such, and if so return it as CredentialData. If neither inputs are valid,
     * we will attempt a best effort normalisation to create valid CredentialData, or abort and return undefined.
     *
     * @public
     * @param data - The credential data to normalise.
     * @param cookie - The unknown cookie data to normalise.
     * @returns A tuple of the best effort normalisation of the inputs to the CredentialData format, and a boolean to indicate if we can remove the cookie.
     */
    normaliseLegacyCredentialData(data?: CredentialData, cookie?: CredentialData): [CredentialData | undefined, boolean];
    /**
     * Stores the specified credential data in the browser's credential manager.
     * @public
     * @param data - The credential data to store.
     * @returns A Promise that resolves to the stored credential object, or `undefined` if storage fails.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CredentialsContainer
     */
    storeCredentialsInContainer(data: CredentialData): Promise<Credential | undefined>;
    /**
     * Retrieves credential data from the browser's credential manager.
     * @public
     * @param options - Options for filtering the retrieved credentials, if any.
     * @returns A Promise that resolves to the retrieved credential data, or `null` if no matching credentials were found.
     */
    getCredentialsFromContainer(options?: GetCredentialOptions): Promise<CredentialData | null>;
    /**
     * Stores the specified credential data in a browser cookie.
     * @public
     * @param data - The credential data to store.
     * @returns `true` if storage was successful, or `false` otherwise.
     */
    storeCredentialsInCookie(data: CredentialData): boolean;
    /**
     * Retrieves credential data from a browser cookie.
     * @public
     * @returns The retrieved credential data, or `undefined` if no matching data was found.
     */
    getCredentialsFromCookie(): CredentialData | undefined;
    /**
     * Stores user credentials securely.
     * @public
     * @param data - The credential data to be stored.
     * @returns A Promise that resolves to a boolean indicating if the credentials were stored successfully.
     */
    storeCredentials(data: CredentialData): Promise<boolean>;
    /**
     * Retrieves stored credentials.
     * @public
     * @param options - An object containing options for retrieving credentials.
     * @returns A Promise that resolves to a CredentialData object if the credentials are found, or undefined otherwise.
     */
    getCredentials(options?: GetCredentialOptions): Promise<CredentialData | undefined>;
    /**
     * Retrieves the current SSO token, if any.
     * @public
     * @returns A string containing the SSO token, or null if there is no SSO token.
     */
    getSSOToken(): string | null;
    /**
     * Retrieves the current username, if any.
     * @public
     * @returns A string containing the username, or null if there is no username.
     */
    getUsername(): string | null;
    /**
     * Retrieves the current refresh token, if any.
     * @public
     * @returns A string containing the refresh token, or null if there is no refresh token.
     */
    getRefreshToken(): string | null;
    /**
     * Removes the user's cookie.
     * @public
     */
    removeCookie(): void;
    /**
     * Removes the current SSO token, if any.
     * @public
     */
    removeSSOToken(): void;
    /**
     * Removes the current username, if any.
     * @public
     */
    removeUsername(): void;
    /**
     * Removes the current refresh token, if any.
     * @public
     */
    removeRefreshToken(): void;
    /**
     * Removes all user crendentials data.
     * @public
     */
    removeAll(): void;
    /**
     * Prevents silent access by requiring the user to authenticate before making API requests.
     * @public
     */
    preventSilentAccess(): Promise<void>;
}
/**
 * The CredentialManager implementation.
 * @internal
 */
export declare class DefaultCredentialManager implements CredentialManager {
    private session;
    private config;
    constructor(session: Session, config?: CredentialManagerConfig);
    createBasicCredentials(inputs: BasicCredentialsInput): BasicAuthInfo;
    createRefreshCredentials(inputs?: RefreshCredentialsInput): RefreshAuthInfo;
    createSSOCredentials(inputs?: SSOCredentialsInput): SsoAuthInfo;
    validateBasicCredentials(credentials: BasicAuthInfo): boolean;
    validateRefreshCredentials(credentials: RefreshAuthInfo): boolean;
    validateSSOCredentials(credentials: SsoAuthInfo): boolean;
    validateUserCredentialData(data: CredentialData, ensurePassword?: boolean): boolean;
    normaliseLegacyCredentialData(data?: CredentialData, cookie?: CredentialData): [CredentialData | undefined, boolean];
    storeCredentialsInContainer(data: CredentialData): Promise<Credential | undefined>;
    getCredentialsFromContainer(options?: CredentialRequestOptions): Promise<CredentialData | null>;
    storeCredentialsInCookie(data: CredentialData): boolean;
    getCredentialsFromCookie(): CredentialData | undefined;
    storeCredentials(data: CredentialData): Promise<boolean>;
    getCredentials(options?: GetCredentialOptions): Promise<CredentialData | undefined>;
    getSSOToken(): string;
    getUsername(): string;
    getRefreshToken(): string;
    removeCookie(): void;
    removeSSOToken(): void;
    removeUsername(): void;
    removeRefreshToken(): void;
    removeAll(): void;
    preventSilentAccess(): Promise<void>;
}
/**
 * The DI token for the CredentialManager interface.
 * @internal
 */
export declare const CredentialManager: import("@microsoft/fast-foundation").InterfaceSymbol<CredentialManager>;
//# sourceMappingURL=credentialManager.d.ts.map