import { Auth } from '../auth';
import { Connect, Message, SocketObservable } from '../connect';
import type { FieldMetadata, FilteredDataServerResult, IndexDetail, MetadataDetail, RequestServerResult } from '../metadata/metadata.types';
import type { DatasourceOptions, Dataview } from './datasource.types';
import { DatasourceConfig, ResourceType } from './datasource.types';
/**
 * The default configuration for the Datasource service.
 *
 * @public
 */
export declare const defaultDatasourceConfig: DatasourceConfig;
/**
 * Datasource interface.
 *
 * @public
 * @remarks
 * The public interface available on the injected store fragment.
 */
export interface Datasource {
    /**
     * Indicates whether the datasource has been initialized.
     * @public
     */
    initialized: boolean;
    /**
     * Metadata about the fields of the datasource.
     * @public
     */
    fieldMetadata: FieldMetadata[];
    /**
     * Indicates whether the datasource is read-only.
     * TODO: Should have is prefix
     * @public
     */
    readOnly: boolean;
    /**
     * The type of resource the datasource represents.
     * @public
     */
    resourceType: ResourceType;
    /**
     * A stream of data from the datasource.
     * @public
     */
    stream: SocketObservable<FilteredDataServerResult | RequestServerResult>;
    /**
     * Initializes the datasource with the given options and fetches metadata if specified.
     * @public
     * @param options - The options to initialize the datasource with.
     * @param fetchMeta - Whether to fetch metadata for the datasource.
     * @returns A promise that resolves with a boolean indicating whether the initialization was successful.
     */
    init(options: DatasourceOptions, fetchMeta: boolean): Promise<boolean>;
    /**
     * Fetches metadata for the specified resource name.
     * @public
     * @param resourceName - The name of the resource to fetch metadata for.
     * @returns A promise that resolves with an array of FieldMetadata.
     */
    getMetadata?(resourceName: string): Promise<Array<FieldMetadata>>;
    /**
     * Takes a snapshot of the data in the datasource.
     * @public
     * @param params - Optional parameters for taking the snapshot.
     */
    snapshot(params?: any): any;
    /**
     * Takes a filtered snapshot of the data in the datasource.
     * @public
     * @param rowId - Optional ID of the row to filter.
     */
    snapshotFiltered(rowId?: string): any;
    /**
     * Updates the data in the datasource with the specified data.
     * @public
     * @param data - The data to update the datasource with.
     * @returns A promise that resolves when the data has been updated.
     */
    updateData?(data: Array<any>): Promise<void>;
    /**
     * Handles the specified data and returns it.
     * @public
     * @param data - The data to handle.
     * @returns The handled data.
     */
    dataHandler?(data: Array<any>): Array<any>;
    /**
     * Adds the specified dataview to the datasource.
     * @public
     * @param view - The dataview to add.
     */
    addView?(view: Dataview): void;
    /**
     * Removes the specified dataview from the datasource.
     * @public
     * @param view - The dataview to remove.
     */
    dropView?(view: Dataview): void;
    /**
     * Sets a filter on the datasource with the specified name, value, and type.
     * @public
     * @param name - The name of the filter to set.
     * @param value - The value of the filter to set.
     * @param type - The type of the filter to set.
     * @returns A promise that resolves when the filter has been set.
     */
    setFilter?(name: string, value: any, type?: string): Promise<void>;
    /**
     * Clears the filter with the specified name from the datasource.
     * @public
     * @param name - The name of the filter to clear.
     * @returns A promise that resolves when the filter has been cleared.
     */
    clearFilter?(name: string): Promise<void>;
    /**
     * Creates a new item in the datasource with the specified data.
     * @public
     * @param data - The data to create the item with.
     * @returns A promise that resolves with the ID of the created item.
     */
    createItem?(data: any): Promise<string>;
    /**
     * Deletes an item with the given ID from the datasource.
     * @public
     * @param itemId - The ID of the item to be deleted.
     * @returns A promise that resolves when the item has been deleted.
     */
    deleteItem?(itemId: string): Promise<void>;
    /**
     * Updates an item in the datasource with the given ID and data.
     * @public
     * @param itemId - The ID of the item to be updated.
     * @param data - The data to be updated for the item.
     * @returns A promise that resolves when the item has been updated.
     */
    updateItem?(itemId: string, data: any): Promise<void>;
}
/**
 * DefaultDatasource.
 *
 * @public
 */
export declare class DefaultDatasource implements Datasource {
    protected connect: Connect;
    protected auth: Auth;
    protected config: DatasourceConfig;
    initialized: boolean;
    readOnly: boolean;
    protected options: DatasourceOptions;
    resourceType: ResourceType;
    protected _stream: SocketObservable<FilteredDataServerResult | RequestServerResult>;
    protected originalFieldDef: MetadataDetail[];
    protected requestFields: MetadataDetail[];
    fieldMetadata: FieldMetadata[];
    protected availableIndexes: IndexDetail[];
    constructor(connect: Connect, auth: Auth, config?: DatasourceConfig);
    init(options: DatasourceOptions, fetchMeta?: boolean): Promise<boolean>;
    private deinit;
    get stream(): SocketObservable<FilteredDataServerResult | RequestServerResult>;
    get params(): any;
    snapshot(): Promise<Message>;
    snapshotFiltered(rowId?: string): Promise<any[]>;
    private createStream;
    private createDataserverStream;
    private createReqRepStream;
    private validResourceName;
}
/**
 * The DI token for the Datasource interface.
 * @internal
 */
export declare const Datasource: import("@microsoft/fast-foundation").InterfaceSymbol<Datasource>;
//# sourceMappingURL=datasource.d.ts.map