import type { FieldMetadata, FilteredDataServerResult, RawDataServerResult } from '../metadata/metadata.types';
/**
 * Types of Genesis resources.
 * @public
 */
export declare enum ResourceType {
    DATASERVER = "DATASERVER",
    EVENT_HANDLER = "EVENT_HANDLER",
    REQUEST_SERVER = "REQUEST_SERVER"
}
/**
 * Data Update definition with details of what data to add, drop, and/or modify.
 * @public
 */
export interface DataUpdate {
    add?: Array<any>;
    drop?: Array<any>;
    modify?: Array<any>;
}
/**
 * Dataview object
 * @public
 */
export interface Dataview {
    /**
     * Function to update the data in the dataview object with an object of optional add, drop, and modify arrays of data.
     * @param data - A DataUpdate object containing the data to add, drop, and/or modify.
     * @returns A promise that resolves when the update is complete.
     */
    update?: (data: DataUpdate) => Promise<void>;
    /**
     * An array of data objects representing the data in the dataview.
     */
    data: any[];
    /**
     * An array of objects representing the metadata of each column in the dataview.
     */
    columnsMetadata?: FieldMetadata[];
    /**
     * A boolean indicating whether the search functionality should be hidden in the UI.
     */
    hideSearch?: boolean;
}
/**
 * Options that can be passed when initializing a datasource.
 * @public
 */
export interface DatasourceOptions {
    /**
     * The criteria used to filter data in the datasource.
     */
    criteria?: string;
    /**
     * The fields to include in the returned data.
     */
    fields?: string;
    /**
     * Whether the datasource is a snapshot or not.
     */
    isSnapshot?: boolean;
    /**
     * The maximum number of rows to return from the datasource.
     */
    maxRows?: number;
    /**
     * The maximum number of rows to track as part of a client "view"
     */
    maxView?: number;
    /**
     * Defines the behaviour of the client view when new rows are received in real time.
     */
    movingView?: boolean;
    /**
     * The field to use for sorting the data.
     */
    orderBy?: string;
    /**
     * The request to send to the server.
     */
    request?: any;
    /**
     * The name of the resource to use for the datasource.
     */
    resourceName?: string;
    /**
     * Whether to return the data in reverse order or not.
     */
    reverse?: boolean;
}
/**
 * Default values for a DatasourceConfig
 * @public
 */
export declare class DatasourceDefaults {
    static readonly MAX_ROWS_250 = 250;
    static readonly MAX_VIEW_1000 = 1000;
    static readonly REQ_REP_POLLING_INTERVAL_MS = 10000;
}
/**
 * The configuration object for a Datasource
 * @public
 */
export interface DatasourceConfig {
    options: DatasourceOptions;
    criteriaToFields(filters: string): Record<string, string>;
    dataServerResultFilter(result: RawDataServerResult, rowId?: string): FilteredDataServerResult;
}
/**
 * The DI token for the DatasourceConfig interface.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const DatasourceConfig: import("@microsoft/fast-foundation").InterfaceSymbol<DatasourceConfig>;
//# sourceMappingURL=datasource.types.d.ts.map